/*******************************************************************************
 * SVMlin
 * Copyright (c) 2006 Vikas Sindhwani at the University of Chicago.
 * Adapted to Percolator by Lukas Käll at the University of Washington
 *
 *******************************************************************************/
#ifndef SSLF_H
#define SSLF_H

#include <vector>

#include "ssl.h"

using namespace std;

#define THREADS (Globals::getInstance()->getNumThreads())
class Reduce_Vectors
{
 public:
  Reduce_Vectors(int size);
  ~Reduce_Vectors();

  void init(void);
  void sum_scale_x(double scalar, double* x);
  void reduce_sum(double* v);
  void reduce_sum_cgls(double* v);

 private:
  int nr_thread;
  int size;
  int size0;
  double **tmp_array;
};

double fun_omp(double *w, int w_size, int l,
           const double* y, double* z, const double* C);

void subXTv_omp(double *v, double *XTv, int w_size,
		double* X, const int* Id, int sizeI, 
		Reduce_Vectors *reduce_vectors);

#ifdef GPUGRAD
void grad_omp(double *w, double *g, int w_size, int l,
	     const double* y, double* z, const double* C, double* X,
	     int* Id, Reduce_Vectors *reduce_vectors, int sizeI);
#else
int grad_omp(double *w, double *g, int w_size, int l,
	     const double* y, double* z, const double* C, double* X,
	     int* Id, Reduce_Vectors *reduce_vectors);
#endif

void Hv_omp(double *s, double *Hs, int w_size,
	    const double* C, double* X, const int *Id, int sizeI,
	    Reduce_Vectors *reduce_vectors);

int omp_thread_count();

#endif
